#include <iostream.h>

struct BoundsError {};

template <class T>
class Range
{
public:
  Range(const T& upper_bound = 0,
	const T& lower_bound = 0);
   // Precondition: upper_bound >= lower_bound
   // Postconditions:
   //   lower == upper_bound
   //   upper == upper_bound
   // Throws:
   //   Bounds error on precondition violation
   //   Whatever T's copy constructor throws.
   //   Whatever operator < on T throws.
  
  const T& lowerBound() throw ();
  const T& upperBound() throw ();
  int includes(const T& aValue);
    // Throws: Whatever operator>= and operator <= on T
    // throws.
private:
  T lower;
  T upper;
};

template <class T>
Range<T>::Range(const T& upper_bound,
                const T& lower_bound)
  : lower(lower_bound), // copy constructor
    upper(upper_bound)  // copy constructor
{
  if (upper < lower) throw BoundsError();
}

template <class T>
const T& Range<T>::lowerBound() throw ()
{
  return lower;  
}

template <class T>
const T& Range<T>::upperBound() throw ()
{
  return upper;
}

template <class T>
int Range<T>::includes(const T& aValue)
{
  return aValue >= lower && aValue <= upper;
}

int main(void)
{
  Range<int> ri(100,10);
  Range<double> rd(3.141592,-3.141592);
  if (ri.includes(55))
  {
    cout << "[" << ri.lowerBound() << ", " 
	 << ri.upperBound() << "] includes 55"
	 << endl;
  }
  if (!rd.includes(62))
  {
    cout << "[" << rd.lowerBound() << ", "
	 << rd.upperBound()
	 << "] does not include 62" << endl;
  }
  return 0;
}

