template <class T>
class type_name
{
public:
  static const char* as_string();
};


const char* type_name<char>::as_string()
{
  return "char";
}

const char* type_name<int>::as_string()
{
  return "int";
}

const char* type_name<unsigned>::as_string()
{
  return "unsigned";
}

const char* type_name<short>::as_string()
{
  return "short";
}

const char* type_name<unsigned short>::as_string()
{
  return "unsigned short";
}

const char* type_name<unsigned char>::as_string()
{
  return "unsigned char";
}

const char* type_name<signed char>::as_string()
{
  return "signed char";
}

const char* type_name<long>::as_string()
{
  return "long";
}

const char* type_name<unsigned long>::as_string()
{
  return "unsigned long";
}

const char* type_name<double>::as_string()
{
  return "double";
}

const char* type_name<float>::as_string()
{
  return "float";
}

struct BoundsError {};

template <class T>
class Range
{
public:
  Range(const T& upper_bound = 0,
	const T& lower_bound = 0);
   // Precondition: upper_bound >= lower_bound
   // Postconditions:
   //   lower == upper_bound
   //   upper == upper_bound
   // Throws:
   //   Bounds error on precondition violation
   //   Whatever T's copy constructor throws.
   //   Whatever operator < on T throws.
  
  const T& lowerBound() const throw ();
  const T& upperBound() const throw ();
  int includes(const T& aValue) const;
    // Throws: Whatever operator>= and operator <= on T
    // throws.
private:
  T lower;
  T upper;
};

template <class T>
Range<T>::Range(const T& upper_bound,
                const T& lower_bound)
  : lower(lower_bound), // copy constructor
    upper(upper_bound)  // copy constructor
{
  if (upper < lower) throw BoundsError();
}

template <class T>
const T& Range<T>::lowerBound() const throw ()
{
  return lower;  
}

template <class T>
const T& Range<T>::upperBound() const throw ()
{
  return upper;
}

template <class T>
int Range<T>::includes(const T& aValue) const
{
  return aValue >= lower && aValue <= upper;
}

template <class T>
Range<T> make_range(const T& t1, const T& t2)
{
  return Range<T>(t1,t2);
}

#include <iostream.h>


template <class T>
void print(const Range<T>& r)
{
  cout << "Range<" << type_name<T>::as_string() << ">(" 
       << r.upperBound() << ", "
       << r.lowerBound() << ")" << endl;
}

int main(void)
{
  print(make_range(10,5));
  print(make_range(3.141592,-2.0));  
}
